#include "DHT.h"
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

// macros
#define TRIG_PIN 11
#define ECHO_PIN 12
#define DHT_PIN 6
#define FAN_PIN_INA 2
#define POT_PIN A0

#define THRESHOLD_DISTANCE 75
#define DELAY_TIME 500
#define MAX_ANALOG 1023
#define MAX_TEMP 35
#define MIN_TEMP 20

// variables
int distance;
float threshold;
float temperature;

DHT dht(DHT_PIN, DHT11);
LiquidCrystal_I2C lcd(0x27,16,2);

void setup() {
  pinMode(TRIG_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  pinMode(FAN_PIN_INA, OUTPUT);
  digitalWrite(FAN_PIN_INA, LOW);

  dht.begin();

  lcd.begin();
  lcd.backlight();
  
  temperature = threshold = 0;
  distance = 0;
}

void calculateDistanceAndTemperature() {
  digitalWrite(TRIG_PIN, LOW);
  delayMicroseconds(2);
  digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  
  digitalWrite(TRIG_PIN, LOW);
  
  long duration = pulseIn(ECHO_PIN, HIGH);
  distance = duration * 0.034 / 2;

  temperature = dht.readTemperature();
}

void calculateThreshold() {
  int threshold_pot = analogRead(POT_PIN);
  threshold = ((float) threshold_pot) * (MAX_TEMP - MIN_TEMP) / MAX_ANALOG + MIN_TEMP;
}

void printOnLCD() {
  lcd.clear();
  lcd.print("Temp: ");
  lcd.print(temperature);
  lcd.print(" C");
  lcd.setCursor(0, 1);
  lcd.print("Prag: ");
  lcd.print(threshold);
  lcd.print(" C");
}

void loop() {
  calculateDistanceAndTemperature();
  for (int i = 0; i < 4; ++i) {
    calculateThreshold();
    if (distance < THRESHOLD_DISTANCE && temperature >= threshold)
      digitalWrite(FAN_PIN_INA, HIGH);
    else
      digitalWrite(FAN_PIN_INA, LOW);
  
    printOnLCD();
    delay(DELAY_TIME);
  }
}
